﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Globalization;
using System.Linq;
using System.Resources;
using System.Text;
using System.Web;
using System.Web.Mvc;


namespace System.Web.Mvc
{
    /// <summary>
    /// Helper extensions methods for ValidationGroups
    /// </summary>
    /// <remarks>This code is based heavily on the Mvc3 implementation to maintain as much of the original functionality as possible (everything AFAIK).</remarks>
    public static class ValidationExtensions
    {

        /// <summary>
        /// Validations the summary.
        /// </summary>
        /// <param name="htmlHelper">The HTML helper.</param>
        /// <param name="groupName">Name of the group.</param>
        /// <param name="excludePropertyErrors">if set to <c>true</c> [exclude property errors].</param>
        /// <returns></returns>
        public static MvcHtmlString ValidationSummary(this HtmlHelper htmlHelper, string groupName, bool excludePropertyErrors)
        {
            return htmlHelper.ValidationSummary(groupName, excludePropertyErrors, string.Empty);
        }

        /// <summary>
        /// Validations the summary.
        /// </summary>
        /// <param name="htmlHelper">The HTML helper.</param>
        /// <param name="groupName">Name of the group.</param>
        /// <param name="excludePropertyErrors">if set to <c>true</c> [exclude property errors].</param>
        /// <param name="message">The message.</param>
        /// <returns></returns>
        public static MvcHtmlString ValidationSummary(this HtmlHelper htmlHelper, string groupName, bool excludePropertyErrors, string message)
        {
            return htmlHelper.ValidationSummary(groupName, excludePropertyErrors, message, null);
        }

        /// <summary>
        /// Validations the summary.
        /// </summary>
        /// <param name="htmlHelper">The HTML helper.</param>
        /// <param name="groupName">Name of the group.</param>
        /// <param name="excludePropertyErrors">if set to <c>true</c> [exclude property errors].</param>
        /// <param name="message">The message.</param>
        /// <param name="htmlAttributes">The HTML attributes.</param>
        /// <returns></returns>
        public static MvcHtmlString ValidationSummary(this HtmlHelper htmlHelper, string groupName, bool excludePropertyErrors, string message, IDictionary<string, object> htmlAttributes)
        {
            #region Validate Arguments
            if (htmlHelper == null) throw new ArgumentNullException("htmlHelper");
            if (string.IsNullOrEmpty(groupName)) throw new ArgumentException("The string cannot be null or empty", groupName);
            #endregion
            if (htmlAttributes == null) htmlAttributes = HtmlHelper.AnonymousObjectToHtmlAttributes(null);
            htmlAttributes["data-val-valgroup-name"] = groupName;


            string[] groupNames = groupName.Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries).Select(s => s.Trim()).ToArray();
            object model = htmlHelper.ViewData.Model;
            ModelStateDictionary modelState = htmlHelper.ViewData.ModelState;

            bool canValidate = !modelState.ContainsKey(ValidationGroupsKey) || (groupNames.Any() && ((IEnumerable<string>)modelState[ValidationGroupsKey].Value.RawValue).Intersect(groupNames).Any());
            IEnumerable<ModelState> faulted = canValidate ? GetFaultedModelStates(htmlHelper, groupNames, model) : Enumerable.Empty<ModelState>();
            FormContext formContextForClientValidation = htmlHelper.ViewContext.ClientValidationEnabled ? htmlHelper.ViewContext.FormContext : null;
            if (!canValidate || !faulted.Any())
            {
                if (formContextForClientValidation == null)
                {
                    return null;
                }
                if (htmlHelper.ViewContext.UnobtrusiveJavaScriptEnabled && excludePropertyErrors)
                {
                    return null;
                }
            }
            string str = InitializeResponseString(message);

            IEnumerable<ModelState> values = GetValues(htmlHelper, excludePropertyErrors, modelState, faulted);
            TagBuilder builder3 = AddValidationMessages(htmlHelper, values);
            TagBuilder tagBuilder = new TagBuilder("div");
            tagBuilder.MergeAttributes<string, object>(htmlAttributes);
            tagBuilder.AddCssClass(!faulted.Any() ? HtmlHelper.ValidationSummaryValidCssClassName : HtmlHelper.ValidationSummaryCssClassName);
            tagBuilder.InnerHtml = str + builder3.ToString(TagRenderMode.Normal);
            if (formContextForClientValidation != null)
            {
                if (htmlHelper.ViewContext.UnobtrusiveJavaScriptEnabled)
                {
                    if (!excludePropertyErrors)
                    {
                        tagBuilder.MergeAttribute("data-valmsg-summary", "true");
                    }
                }
                else
                {
                    tagBuilder.GenerateId("validationSummary");
                    formContextForClientValidation.ValidationSummaryId = tagBuilder.Attributes["id"];
                    formContextForClientValidation.ReplaceValidationSummary = !excludePropertyErrors;
                }
            }
            return new MvcHtmlString(tagBuilder.ToString(TagRenderMode.Normal));

        }

        #region Validation Methods Based (Heavily) on MVC3 Implementation

        private static TagBuilder AddValidationMessages(HtmlHelper htmlHelper, IEnumerable<ModelState> values)
        {
            StringBuilder builder2 = new StringBuilder();
            TagBuilder builder3 = new TagBuilder("ul");
            if (values != null)
            {
                foreach (ModelState state2 in values)
                {
                    foreach (ModelError error in state2.Errors)
                    {
                        string str2 = GetUserErrorMessageOrDefault(htmlHelper.ViewContext.HttpContext, error, null);
                        if (!string.IsNullOrEmpty(str2))
                        {
                            TagBuilder builder4 = new TagBuilder("li");
                            builder4.SetInnerText(str2);
                            builder2.AppendLine(builder4.ToString(TagRenderMode.Normal));
                        }
                    }
                }
            }
            if (builder2.Length == 0)
            {
                builder2.AppendLine("<li style=\"display:none\"></li>");
            }
            builder3.InnerHtml = builder2.ToString();
            return builder3;
        }

        private static IEnumerable<ModelState> GetValues(HtmlHelper htmlHelper, bool excludePropertyErrors, ModelStateDictionary modelState, IEnumerable<ModelState> faulted)
        {
            IEnumerable<ModelState> values = null;
            if (excludePropertyErrors)
            {
                ModelState state;
                modelState.TryGetValue(htmlHelper.ViewData.TemplateInfo.HtmlFieldPrefix, out state);
                if (state != null)
                {
                    values = new ModelState[] { state };
                }
            }
            else
            {
                values = faulted;
            }
            return values;
        }

        private static string InitializeResponseString(string message)
        {
            string str;
            if (!string.IsNullOrEmpty(message))
            {
                TagBuilder builder = new TagBuilder("span");
                builder.SetInnerText(message);
                str = builder.ToString(TagRenderMode.Normal) + Environment.NewLine;
            }
            else
            {
                str = null;
            }
            return str;
        }

        private static IEnumerable<ModelState> GetFaultedModelStates(HtmlHelper htmlHelper, string[] groupNames, object model)
        {
            IEnumerable<string> validatedPropertyNames = TypeDescriptor.GetProperties(model).OfType<PropertyDescriptor>()
                .Where(pd => pd.Attributes.OfType<ValidationGroupAttribute>()
                    .Where(vg => groupNames.Intersect(vg.GroupNames, StringComparer.OrdinalIgnoreCase).Any()).Any()
                ).Select(pd => pd.Name).ToArray();

            IEnumerable<ModelState> faulted = htmlHelper.ViewData.ModelState.Where(ms => validatedPropertyNames.Contains(ms.Key) && !htmlHelper.ViewData.ModelState.IsValidField(ms.Key)).Select(ms => ms.Value).ToArray();
            return faulted;
        }

        /// <summary>
        /// Validations the summary.
        /// </summary>
        /// <param name="htmlHelper">The HTML helper.</param>
        /// <param name="groupName">Name of the group.</param>
        /// <param name="excludePropertyErrors">if set to <c>true</c> [exclude property errors].</param>
        /// <param name="message">The message.</param>
        /// <param name="htmlAttributes">The HTML attributes.</param>
        /// <returns></returns>
        public static MvcHtmlString ValidationSummary(this HtmlHelper htmlHelper, string groupName, bool excludePropertyErrors, string message, object htmlAttributes)
        {
            return htmlHelper.ValidationSummary(groupName, excludePropertyErrors, message, HtmlHelper.AnonymousObjectToHtmlAttributes(htmlAttributes));
        }


        #region Extracted ValidationSummary Helper Methods

        private static string GetUserErrorMessageOrDefault(HttpContextBase httpContext, ModelError error, ModelState modelState)
        {
            if (!string.IsNullOrEmpty(error.ErrorMessage))
            {
                return error.ErrorMessage;
            }
            if (modelState == null)
            {
                return null;
            }
            string str = (modelState.Value != null) ? modelState.Value.AttemptedValue : null;
            return string.Format(CultureInfo.CurrentCulture, GetInvalidPropertyValueResource(httpContext), new object[] { str });
        }

        private static string GetInvalidPropertyValueResource(HttpContextBase httpContext)
        {
            string str = null;
            if (!string.IsNullOrEmpty(ResourceClassKey) && (httpContext != null))
            {
                str = httpContext.GetGlobalResourceObject(ResourceClassKey, "InvalidPropertyValue", CultureInfo.CurrentUICulture) as string;
            }
            return (str ?? Common_ValueNotValidForProperty);
        }

        #endregion

        #region Extracted Required Methods from MvcResources

        internal static string Common_ValueNotValidForProperty
        {
            get
            {
                return ResourceManager.GetString("Common_ValueNotValidForProperty", CultureInfo.CurrentUICulture);
            }
        }


        /// <summary>
        /// Gets the resource class key.
        /// </summary>
        public static string ResourceClassKey
        {
            get
            {
                return ValidationExtensions.ResourceClassKey;
            }

        }

        private static ResourceManager resourceMan;


        [EditorBrowsable(EditorBrowsableState.Advanced)]
        internal static ResourceManager ResourceManager
        {
            get
            {
                if (object.ReferenceEquals(resourceMan, null))
                {
                    ResourceManager manager = new ResourceManager("System.Web.Mvc.Resources.MvcResources", typeof(System.Web.Mvc.Controller).Assembly);
                    resourceMan = manager;
                }
                return resourceMan;
            }
        }

        #endregion

        private static readonly string ValidationGroupsKey = "ValidationGroups_" + Guid.NewGuid().ToString();

        /// <summary>
        /// Determines whether [is group valid] [the specified model state].
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="modelState">State of the model.</param>
        /// <param name="model">The model.</param>
        /// <param name="groupName">Name of the group.</param>
        /// <returns>
        ///   <c>true</c> if [is group valid] [the specified model state]; otherwise, <c>false</c>.
        /// </returns>
        public static bool IsGroupValid<T>(this ModelStateDictionary modelState, T model, string groupName)
        {
            if (modelState == null) throw new ArgumentNullException("modelState");
            if (model == null) throw new ArgumentNullException("model");
            if (string.IsNullOrEmpty(groupName)) return modelState.IsValid;
            return modelState.IsGroupValid(model, groupName.Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries).Select(s => s.Trim()));
        }

        /// <summary>
        /// Determines whether [is group valid] [the specified model state].
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="modelState">State of the model.</param>
        /// <param name="model">The model.</param>
        /// <param name="groupNames">The group names.</param>
        /// <returns>
        ///   <c>true</c> if [is group valid] [the specified model state]; otherwise, <c>false</c>.
        /// </returns>
        public static bool IsGroupValid<T>(this ModelStateDictionary modelState, T model, IEnumerable<string> groupNames)
        {
            if (modelState == null) throw new ArgumentNullException("modelState");
            if (model == null) throw new ArgumentNullException("model");
            if (groupNames == null || !groupNames.Any()) return modelState.IsValid;
            modelState.Add(ValidationGroupsKey, new ModelState { Value = new ValueProviderResult(groupNames, string.Join(" ", groupNames), CultureInfo.CurrentUICulture) });
            IEnumerable<string> populated = groupNames
                .Where(s => !string.IsNullOrEmpty(s))
                .Select(s => s.Trim())
                .ToArray();
            if (!populated.Any()) return modelState.IsValid;
            IEnumerable<PropertyDescriptor> props = GetProperties((model == null) ? typeof(T) : model.GetType(), model);
            IEnumerable<PropertyDescriptor> groupProperties = props.Where(pd => pd.Attributes.OfType<ValidationGroupAttribute>()
                .Where(vg => groupNames.Intersect(vg.GroupNames, StringComparer.OrdinalIgnoreCase).Any()).Any()).ToArray();
            if (!groupProperties.Any())
                throw new NotSupportedException(string.Format(CultureInfo.InvariantCulture, "There are no model properties defined as part of validation group '{0}'", string.Join("' or '", groupNames.ToArray())));
            foreach (PropertyDescriptor prop in props.Except(groupProperties))
            {
                if (modelState[prop.Name] == null) continue;
                modelState[prop.Name].Errors.Clear();
            }
            return groupProperties.All<PropertyDescriptor>(pd => modelState[pd.Name] != null
                && modelState[pd.Name].Errors != null
                && !modelState[pd.Name].Errors.Any());

        }

        private static IEnumerable<PropertyDescriptor> GetProperties(Type valueType, object value)
        {
            IEnumerable<PropertyDescriptor> props = Enumerable.Empty<PropertyDescriptor>();
            if (value == null)
            {
                props = TypeDescriptor.GetProperties(valueType).OfType<PropertyDescriptor>().ToArray();
            }
            else
            {
                props = TypeDescriptor.GetProperties(value).OfType<PropertyDescriptor>().ToArray();
            }
            foreach (var p in props.Where(p => p != null))
            {
                yield return p;
                if (p.PropertyType != null
                    && p.PropertyType.IsClass
                    && !p.PropertyType.Namespace.StartsWith("System", StringComparison.OrdinalIgnoreCase)
                    && !p.PropertyType.Namespace.StartsWith("Microsoft", StringComparison.OrdinalIgnoreCase))
                {
                    foreach (var p2 in GetProperties(p.PropertyType, (value == null) ? null : p.GetValue(value)))
                    {
                        yield return p2;
                    }
                }
            }
            yield break;
        }

        #endregion
    }
}
